@@ -18,6 +18,7 @@ pub use quality::left as refl_left;
 pub use quality::right as refl_right;
 
 use existence::EProp;
+use univalence::Hom;
 use nat::*;
 
 /// Models a type relation `a : t`.
@@ -33,6 +34,74 @@ pub fn ty_in_right_arg<A: Prop, B: Prop, C: Prop>((ab, pord): Ty<A, B>, eq: Eq<B
     (imply::in_right_arg(ab, eq.clone()), pord.by_eq_right(eq))
 }
 
+/// `(x : a) ⋀ (y : b)  =>  ((x ⋀ y) : (a ⋀ b))`.
+pub fn ty_and<X: Prop, Y: Prop, A: Prop, B: Prop>(
+    (xa, pord_xa): Ty<X, A>,
+    (yb, pord_yb): Ty<Y, B>,
+) -> Ty<And<X, Y>, And<A, B>> {
+    let imply_and_xy_and_ab: Imply<And<X, Y>, And<A, B>> = Rc::new(move |(x, y)| {
+        let a = xa(x);
+        let b = yb(y);
+        let and_ab = (a, b);
+        and_ab
+    });
+    (imply_and_xy_and_ab, pord_xa.and(pord_yb))
+}
+
+/// `(x : a) ⋀ (y : b)  =>  ((x ⋁ y) : (a ⋁ b))`.
+pub fn ty_or<X: Prop, Y: Prop, A: Prop, B: Prop>(
+    (xa, pord_xa): Ty<X, A>,
+    (yb, pord_yb): Ty<Y, B>,
+) -> Ty<Or<X, Y>, Or<A, B>> {
+    let or_xy_or_ab: Imply<Or<X, Y>, Or<A, B>> = Rc::new(move |or_xy| {
+        match or_xy {
+            Left(x) => Left(xa(x)),
+            Right(y) => Right(yb(y)),
+        }
+    });
+    (or_xy_or_ab, pord_xa.or(pord_yb))
+}
+
+/// `(x : a) ⋀ (y : b) ⋀ hom(x, y)  =>  ((x => y) : (a => b))`.
+pub fn ty_hom_imply<X: Prop, Y: Prop, A: Prop, B: Prop>(
+    (xa, pord_xa): Ty<X, A>,
+    (yb, pord_yb): Ty<Y, B>,
+    hom: Hom<X, Y>,
+) -> Ty<Imply<X, Y>, Imply<A, B>> {
+    let pord = pord_xa.clone().imply(pord_yb.clone());
+    let xy_ab = Rc::new(move |xy| {
+        let q_xy = hom(xy);
+        let psem = assume();
+        let q_ab = psem(((q_xy, (pord_xa.clone(), pord_yb.clone())), (xa.clone(), yb.clone())));
+        quality::to_eq(q_ab).0
+    });
+    (xy_ab, pord)
+}
+
+/// `(x : a) ⋀ (y : b) ⋀ eqq(x, y)  =>  ((x => y) : (a => b))`.
+pub fn ty_eqq_imply<X: DProp, Y: DProp, A: Prop, B: Prop>(
+    (xa, pord_xa): Ty<X, A>,
+    (yb, pord_yb): Ty<Y, B>,
+    eqq_xy: EqQ<X, Y>,
+) -> Ty<Imply<X, Y>, Imply<A, B>> {
+    let pord = pord_xa.clone().imply(pord_yb.clone());
+    let xy_ab: Imply<Imply<X, Y>, Imply<A, B>> = match (X::decide(), Y::decide()) {
+        (Left(x), _) => Rc::new(move |xy| yb(xy(x.clone())).map_any()),
+        (_, Left(y)) => {
+            let ab: Imply<A, B> = yb(y).map_any();
+            ab.map_any()
+        }
+        (Right(nx), Right(ny)) => {
+            let eq_xy: Eq<X, Y> = eq::rev_modus_tollens(and::to_eq_pos((ny, nx)));
+            let q_xy = eqq_xy(eq_xy);
+            let q_ab = assume()(((q_xy, (pord_xa, pord_yb)), (xa, yb)));
+            let ab = quality::to_eq(q_ab).0;
+            ab.map_any()
+        }
+    };
+    (xy_ab, pord)
+}
+
 /// Core axiom of Path Semantics.
 pub type PSem<F1, F2, X1, X2> = Imply<
     And<And<Q<F1, F2>, And<POrdProof<F1, X1>, POrdProof<F2, X2>>>,
@@ -121,6 +190,31 @@ impl<T, U> POrdProof<T, U> {
     pub fn by_imply_right<V>(self, _: Imply<U, V>) -> POrdProof<T, V> {
         POrdProof(std::marker::PhantomData)
     }
+
+    /// Merges two proofs of order at right side.
+    pub fn merge_right<V>(self, _: POrdProof<T, V>) -> POrdProof<T, And<U, V>> {
+        POrdProof(std::marker::PhantomData)
+    }
+
+    /// Merges two proofs of order at left side.
+    pub fn merge_left<V>(self, _: POrdProof<V, U>) -> POrdProof<And<T, V>, U> {
+        POrdProof(std::marker::PhantomData)
+    }
+
+    /// Combine two proofs into one using AND.
+    pub fn and<T2, U2>(self, _: POrdProof<T2, U2>) -> POrdProof<And<T, T2>, And<U, U2>> {
+        POrdProof(std::marker::PhantomData)
+    }
+
+    /// Combine two proofs into one using OR.
+    pub fn or<T2, U2>(self, _: POrdProof<T2, U2>) -> POrdProof<Or<T, T2>, Or<U, U2>> {
+        POrdProof(std::marker::PhantomData)
+    }
+
+    /// Combine two proofs into one using IMPLY.
+    pub fn imply<T2, U2>(self, _: POrdProof<T2, U2>) -> POrdProof<Imply<T, T2>, Imply<U, U2>> {
+        POrdProof(std::marker::PhantomData)
+    }
 }
 
 /// Path semantical order.
@@ -574,6 +668,36 @@ pub fn lt_lev<A: LProp, B: LProp>(_a: A, _b: B) where A::N: Lt<B::N> {}
 /// Checks that `X` is qual to `T`.
 pub fn check_q<T, X>(_: Q<X, T>) {}
 
+/// Creation theorem.
+///
+/// For more information, see [paper](https://github.com/advancedresearch/path_semantics/blob/master/papers-wip2/creation-theorem.pdf).
+pub fn creation<X: DProp, Y: DProp, A: Prop, B: Prop>(
+    nx: Not<X>,
+    ty_y_b: Ty<Y, B>,
+    eqq_xy: EqQ<X, Y>,
+    pord: POrdProof<X, A>,
+) -> Imply<A, B> {
+    let nx2 = nx.clone();
+    let ty_x_a: Ty<X, A> = (Rc::new(move |x| not::absurd(nx2.clone(), x)), pord);
+    let xy_ab = ty_eqq_imply(ty_x_a, ty_y_b, eqq_xy);
+    let xy: Imply<X, Y> = Rc::new(move |x| not::absurd(nx.clone(), x));
+    xy_ab.0(xy)
+}
+
+/// Creation theorem using homotopy map.
+pub fn creation_hom<X: Prop, Y: Prop, A: Prop, B: Prop>(
+    nx: Not<X>,
+    ty_y_b: Ty<Y, B>,
+    hom_xy: Hom<X, Y>,
+    pord: POrdProof<X, A>,
+) -> Imply<A, B> {
+    let nx2 = nx.clone();
+    let ty_x_a: Ty<X, A> = (Rc::new(move |x| not::absurd(nx2.clone(), x)), pord);
+    let xy_ab = ty_hom_imply(ty_x_a, ty_y_b, hom_xy);
+    let xy: Imply<X, Y> = Rc::new(move |x| not::absurd(nx.clone(), x));
+    xy_ab.0(xy)
+}
+
 #[cfg(test)]
 #[allow(dead_code)]
 mod test {
